@@ -15,15 +15,19 @@ use serde::{Deserialize, Serialize};
 use serde_json::json;
 use chrono::{DateTime, Utc};
 use poem::web::cookie::Cookie;
+use crate::auth::Role;
 use super::{mail, Permission, user::{User, UserChangeset}, user_session::{UserSession, UserSessionChangeset}, Auth, PaginationParams, ID, UTC, AccessTokenClaims, UserSessionResponse, UserSessionJson};
 
 const COOKIE_NAME: &'static str = "request_token";
 
+type Seconds = i64;
+
 #[derive(Deserialize, Serialize)]
 struct LoginInput {
     email: String,
     password: String,
     device: Option<String>,
+    ttl: Option<Seconds>
 }
 
 #[derive(Debug, Serialize, Deserialize)]
@@ -191,11 +195,13 @@ async fn login(
     // verify device
     let mut device = None;
     if item.device.is_some() {
-        device = match item.device.unwrap().as_str() {
-            "web" => Some("web".to_string()),
-            "mobile" => Some("mobile".to_string()),
-            _ => None,
-        };
+        let device_string = item.device.unwrap();
+        if device_string.len() > 256 {
+            return Err(Error::from_string(
+                "'device' cannot be longer than 256 characters.",
+                StatusCode::BAD_REQUEST,
+            ));
+        }
     }
 
     let user = User::find_by_email(&db, item.email);
@@ -234,7 +240,7 @@ async fn login(
         ));
     }
 
-    let permissions = Permission::for_user(&db, user.id);
+    let permissions = Permission::fetch_all(&db, user.id);
     if permissions.is_err() {
         println!("{:#?}", permissions.err());
         return Err(Error::from_string(
@@ -244,10 +250,30 @@ async fn login(
     }
     let permissions = permissions.unwrap();
 
+    let roles = Role::fetch_all(&db, user.id);
+    if roles.is_err() {
+        println!("{:#?}", roles.err());
+        return Err(Error::from_string(
+            "An internal server error occurred.",
+            StatusCode::INTERNAL_SERVER_ERROR,
+        ));
+    }
+    let roles = roles.unwrap();
+
+    let access_token_duration = chrono::Duration::seconds(
+        if item.ttl.is_some() {
+            std::cmp::max(item.ttl.unwrap(), 1)
+        } else {
+            /* 15 minutes */
+            15 * 60
+        }
+    );
+
     let access_token_claims = AccessTokenClaims {
-        exp: (chrono::Utc::now() + chrono::Duration::minutes(15)).timestamp() as usize,
+        exp: (chrono::Utc::now() + access_token_duration).timestamp() as usize,
         sub: user.id,
         token_type: "access_token".to_string(),
+        roles,
         permissions,
     };
 
@@ -348,7 +374,9 @@ async fn logout(db: Data<&Database>, req: &Request) -> Result<impl IntoResponse>
     let mut builder = Response::builder().status(StatusCode::OK);
 
     if let Some(ref cookie) = refresh_token_cookie {
-        jar.remove(cookie.name());
+        let mut removal_cookie = cookie.clone();
+        removal_cookie.make_removal();
+        jar.add(removal_cookie);
     }
 
     Ok(builder.finish())
@@ -417,7 +445,7 @@ async fn refresh(db: Data<&Database>, req: &Request) -> Result<impl IntoResponse
 
     let session = session.unwrap();
 
-    let permissions = Permission::for_user(&db, session.user_id);
+    let permissions = Permission::fetch_all(&db, session.user_id);
     if permissions.is_err() {
         return Ok(Response::builder()
             .status(StatusCode::INTERNAL_SERVER_ERROR)
@@ -425,10 +453,19 @@ async fn refresh(db: Data<&Database>, req: &Request) -> Result<impl IntoResponse
     }
     let permissions = permissions.unwrap();
 
+    let roles = Role::fetch_all(&db, session.user_id);
+    if roles.is_err() {
+        return Ok(Response::builder()
+            .status(StatusCode::INTERNAL_SERVER_ERROR)
+            .body("{ \"message\": \"An internal server error occurred.\" }"));
+    }
+    let roles = roles.unwrap();
+
     let access_token_claims = AccessTokenClaims {
         exp: (chrono::Utc::now() + chrono::Duration::minutes(15)).timestamp() as usize,
         sub: session.user_id,
         token_type: "access_token".to_string(),
+        roles,
         permissions,
     };
 
@@ -862,12 +899,18 @@ async fn reset_password(
     ))
 }
 
+#[handler]
+async fn check(auth: Auth) -> Result<impl IntoResponse> {
+    Ok(Response::builder().status(StatusCode::OK).finish())
+}
+
 pub fn api() -> Route {
     Route::new()
         .at("/sessions", get(sessions).delete(destroy_sessions))
         .at("/sessions/:id", delete(destroy_session))
         .at("/login", post(login))
         .at("/logout", post(logout))
+        .at("/check", post(check))
         .at("/refresh", post(refresh))
         .at("/register", post(register))
         .at("/activate", get(activate))
