@@ -7,9 +7,6 @@ compile_error!("feature \"backend_actix-web\" and feature \"backend_poem\" canno
 #[macro_use]
 extern crate diesel;
 
-#[cfg(feature = "plugin_auth")]
-pub mod schema;
-
 #[cfg(feature = "plugin_auth")]
 pub mod auth;
 
@@ -17,13 +14,18 @@ pub mod auth;
 pub mod dev;
 
 mod database;
-pub use database::{Database, Connection};
+pub use database::{Database, Pool, Connection};
 
 #[cfg(feature = "backend_poem")]
 mod logger;
 #[cfg(feature = "backend_poem")]
 pub use logger::Logger as PoemLogger;
 
+#[cfg(feature = "plugin_storage")]
+mod storage;
+#[cfg(feature = "plugin_storage")]
+pub use storage::{Storage, Attachment, AttachmentData, AttachmentBlob};
+
 mod mailer;
 pub use mailer::Mailer;
 
@@ -35,6 +37,8 @@ pub use mailer::Mailer;
 pub struct AppData {
     pub mailer: Mailer,
     pub database: Database,
+    #[cfg(feature= "plugin_storage")]
+    pub storage: storage::Storage
 }
 
 pub fn setup() -> AppData {
@@ -50,11 +54,22 @@ pub fn setup() -> AppData {
         // diesel_migrations::embed_migrations!();
     }
 
+    #[cfg(feature="plugin_auth")]
+    if std::env::var("SECRET_KEY").is_err() {
+        panic!("No SECRET_KEY environment variable set!");
+    }
+
+    if std::env::var("DATABASE_URL").is_err() {
+        panic!("No DATABASE_URL environment variable set!");
+    }
+
     Mailer::check_environment_variables();
 
     AppData {
         mailer: Mailer::new(),
         database: Database::new(),
+        #[cfg(feature= "plugin_storage")]
+        storage: storage::Storage::new()
     }
 }
 
@@ -63,7 +78,9 @@ pub fn setup() -> AppData {
 use actix_web;
 
 #[cfg(feature = "backend_actix-web")]
-pub const not_found: fn() -> actix_http::Response = || actix_web::HttpResponse::NotFound().finish();
+pub async fn not_found() -> actix_web::HttpResponse {
+    actix_web::HttpResponse::NotFound().finish()
+}
 
 #[cfg(feature = "backend_poem")]
 use poem;
